import os
import nose
import shutil
import yaml
from fundamentals.utKit import utKit

from fundamentals import tools

su = tools(
    arguments={"settingsFile": None},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName="fundamentals",
    tunnel=False
)
arguments, settings, log, dbConn = su.setup()

# load settings
stream = file(
    "/Users/Dave/.config/fundamentals/fundamentals.yaml", 'r')
settings = yaml.load(stream)
stream.close()

# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()


class test_table_exists():

    def test_table_exists_function(self):

        from fundamentals.mysql import writequery
        sqlQuery = "CREATE TABLE `testing_table` (`id` INT NOT NULL, PRIMARY KEY (`id`))"
        writequery(
            log=log,
            sqlQuery=sqlQuery,
            dbConn=dbConn,
            Force=False,
            manyValueList=False
        )

        from fundamentals.mysql import table_exists
        tableName = "testing_table"
        this = table_exists(
            dbConn=dbConn,
            log=log,
            dbTableName=tableName
        )
        print "%(tableName)s exists: %(this)s" % locals()

        from fundamentals.mysql import writequery
        sqlQuery = "DROP TABLE `testing_table`;"
        writequery(
            log=log,
            sqlQuery=sqlQuery,
            dbConn=dbConn,
            Force=False,
            manyValueList=False
        )

    def test_table_exists_function02(self):

        from fundamentals.mysql import table_exists
        tableName = "stupid_named_table"
        this = table_exists(
            dbConn=dbConn,
            log=log,
            dbTableName=tableName
        )
        print "%(tableName)s exists: %(this)s" % locals()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Render a python list of dictionaries in various list and markup formats*

:Author:
    David Young

:Date Created:
    September 14, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
import io
import csv
import codecs
import copy
import json
import yaml
from decimal import Decimal
from datetime import datetime
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from fundamentals.mysql import convert_dictionary_to_mysql_table


class list_of_dictionaries():
    """
    *The dataset object is a list of python dictionaries. Using this class, the data can be rendered as various list and markup formats*

    **Key Arguments:**
        - ``log`` -- logger
        - ``listOfDictionaries`` -- the list of dictionaries to render

    **Usage:**

        To initialise the dataset object:

        .. code-block:: python

            dataList = [
                {
                    "owner": "daisy",
                    "pet": "dog",
                    "address": "belfast, uk"
                },
                {
                    "owner": "john",
                    "pet": "snake",
                    "address": "the moon"
                },
                {
                    "owner": "susan",
                    "pet": "crocodile",
                    "address": "larne"
                }

            ]

            from fundamentals.renderer import list_of_dictionaries
            dataSet = list_of_dictionaries(
                log=log,
                listOfDictionaries=dataList
            )
    """

    def __init__(
            self,
            log,
            listOfDictionaries,
    ):
        self.log = log
        self.log.debug("instansiating a new 'list_of_dictionaries' object")
        self.listOfDictionaries = listOfDictionaries

        return None

    def csv(
        self,
        filepath=None
    ):
        """*Render the data in CSV format*

        **Key Arguments:**
            - ``filepath`` -- path to the file to write the csv content to. Default *None*

        **Return:**
            - ``renderedData`` -- the data rendered in csv format

        **Usage:**

            To render the data set as csv:

            .. code-block:: python

                print dataSet.csv()

            .. code-block:: text

                owner,pet,address
                daisy,dog,"belfast, uk"
                john,snake,the moon
                susan,crocodile,larne

            and to save the csv rendering to file:

            .. code-block:: python

                dataSet.csv("/path/to/myfile.csv")
        """
        self.log.info('starting the ``csv`` method')

        renderedData = self._list_of_dictionaries_to_csv("machine")

        if filepath and renderedData != "NO MATCH":

            writeFile = codecs.open(filepath, encoding='utf-8', mode='w')
            writeFile.write(renderedData)
            writeFile.close()

        self.log.info('completed the ``csv`` method')
        return renderedData

    def table(
        self,
        filepath=None
    ):
        """*Render the data as a  plain text table*

        **Key Arguments:**
            - ``filepath`` -- path to the file to write the table to. Default *None*

        **Return:**
            - ``renderedData`` -- the data rendered as a plain text table

        **Usage:**

            To render the data set as a plain text table:

            .. code-block:: python

                print dataSet.table()

            .. code-block:: text

                +--------+------------+--------------+
                | owner  | pet        | address      |
                +--------+------------+--------------+
                | daisy  | dog        | belfast, uk  |
                | john   | snake      | the moon     |
                | susan  | crocodile  | larne        |
                +--------+------------+--------------+

            and to save the table rendering to file:

            .. code-block:: python

                dataSet.table("/path/to/myfile.ascii")
        """
        self.log.info('starting the ``table`` method')

        self.filepath = filepath
        renderedData = self._list_of_dictionaries_to_csv("human")

        if filepath and len(self.listOfDictionaries):

            writeFile = codecs.open(filepath, encoding='utf-8', mode='w')
            writeFile.write(renderedData)
            writeFile.close()

        self.log.info('completed the ``table`` method')
        return renderedData

    def markdown(
        self,
        filepath=None
    ):
        """*Render the data as a markdown table*

        **Key Arguments:**
            - ``filepath`` -- path to the file to write the markdown to. Default *None*

        **Return:**
            - ``renderedData`` -- the data rendered as a markdown table

        **Usage:**

            To render the data set as a markdown table:

            .. code-block:: python

                print dataSet.markdown()

            .. code-block:: markdown

                | owner  | pet        | address      |
                |:-------|:-----------|:-------------|
                | daisy  | dog        | belfast, uk  |
                | john   | snake      | the moon     |
                | susan  | crocodile  | larne        |

            and to save the markdown table rendering to file:

            .. code-block:: python

                dataSet.table("/path/to/myfile.md")
        """
        self.log.info('starting the ``markdown`` method')

        self.filepath = filepath
        renderedData = self._list_of_dictionaries_to_csv("markdown")

        if filepath and len(self.listOfDictionaries):

            writeFile = codecs.open(filepath, encoding='utf-8', mode='w')
            writeFile.write(renderedData)
            writeFile.close()

        self.log.info('completed the ``markdown`` method')
        return renderedData

    def json(
        self,
        filepath=None
    ):
        """*Render the data in json format*

        **Key Arguments:**
            - ``filepath`` -- path to the file to write the json content to. Default *None*

        **Return:**
            - ``renderedData`` -- the data rendered as json

        **Usage:**

            To render the data set as json:

            .. code-block:: python

                print dataSet.json()

            .. code-block:: json

                [
                    {
                        "address": "belfast, uk",
                        "owner": "daisy",
                        "pet": "dog"
                    },
                    {
                        "address": "the moon",
                        "owner": "john",
                        "pet": "snake"
                    },
                    {
                        "address": "larne",
                        "owner": "susan",
                        "pet": "crocodile"
                    }
                ]

            and to save the json rendering to file:

            .. code-block:: python

                dataSet.json("/path/to/myfile.json")
        """
        self.log.info('starting the ``json`` method')

        dataCopy = copy.deepcopy(self.listOfDictionaries)
        renderedData = json.dumps(
            dataCopy,
            separators=(',', ': '),
            sort_keys=True,
            indent=4
        )

        if filepath and len(self.listOfDictionaries):

            writeFile = codecs.open(filepath, encoding='utf-8', mode='w')
            writeFile.write(renderedData)
            writeFile.close()

        self.log.info('completed the ``json`` method')
        return renderedData

    def yaml(
        self,
        filepath=None
    ):
        """*Render the data in yaml format*

        **Key Arguments:**
            - ``filepath`` -- path to the file to write the yaml content to. Default *None*

        **Return:**
            - ``renderedData`` -- the data rendered as yaml

        **Usage:**

            To render the data set as yaml:

            .. code-block:: python

                print dataSet.yaml()

            .. code-block:: yaml

                - address: belfast, uk
                  owner: daisy
                  pet: dog
                - address: the moon
                  owner: john
                  pet: snake
                - address: larne
                  owner: susan
                  pet: crocodile

            and to save the yaml rendering to file:

            .. code-block:: python

                dataSet.json("/path/to/myfile.yaml")
        """
        self.log.info('starting the ``yaml`` method')

        dataCopy = []
        dataCopy[:] = [dict(l) for l in self.listOfDictionaries]
        renderedData = yaml.dump(dataCopy, default_flow_style=False)

        if filepath and len(self.listOfDictionaries):

            stream = file(filepath, 'w')
            yaml.dump(dataCopy, stream, default_flow_style=False)
            stream.close()

        self.log.info('completed the ``yaml`` method')
        return renderedData

    def mysql(
        self,
        tableName,
        filepath=None,
        createStatement=None
    ):
        """*Render the dataset as a series of mysql insert statements*

        **Key Arguments:**
            - ``tableName`` -- the name of the mysql db table to assign the insert statements to.
            - ``filepath`` -- path to the file to write the mysql inserts content to. Default *None*
            createStatement

        **Return:**
            - ``renderedData`` -- the data rendered mysql insert statements (string format)

        **Usage:**

            .. code-block:: python

                print dataSet.mysql("testing_table")

            this output the following:

            .. code-block:: plain

                INSERT INTO `testing_table` (address,dateCreated,owner,pet) VALUES ("belfast, uk" ,"2016-09-14T16:21:36" ,"daisy" ,"dog")  ON DUPLICATE KEY UPDATE  address="belfast, uk", dateCreated="2016-09-14T16:21:36", owner="daisy", pet="dog" ;
                INSERT INTO `testing_table` (address,dateCreated,owner,pet) VALUES ("the moon" ,"2016-09-14T16:21:36" ,"john" ,"snake")  ON DUPLICATE KEY UPDATE  address="the moon", dateCreated="2016-09-14T16:21:36", owner="john", pet="snake" ;
                INSERT INTO `testing_table` (address,dateCreated,owner,pet) VALUES ("larne" ,"2016-09-14T16:21:36" ,"susan" ,"crocodile")  ON DUPLICATE KEY UPDATE  address="larne", dateCreated="2016-09-14T16:21:36", owner="susan", pet="crocodile" ;

            To save this rendering to file use:

            .. code-block:: python

                dataSet.mysql("testing_table", "/path/to/myfile.sql")

        """
        self.log.info('starting the ``csv`` method')

        import re
        if createStatement and "create table if not exists" not in createStatement.lower():
            regex = re.compile(r'^\s*CREATE TABLE ', re.I | re.S)
            createStatement = regex.sub(
                "CREATE TABLE IF NOT EXISTS ", createStatement)

        renderedData = self._list_of_dictionaries_to_mysql_inserts(
            tableName=tableName,
            createStatement=createStatement
        )

        if filepath and len(self.listOfDictionaries):

            writeFile = codecs.open(filepath, encoding='utf-8', mode='w')
            writeFile.write(renderedData)
            writeFile.close()

        self.log.info('completed the ``csv`` method')
        return renderedData

    def _list_of_dictionaries_to_csv(
            self,
            csvType="human"):
        """Convert a python list of dictionaries to pretty csv output

        **Key Arguments:**
            - ``csvType`` -- human or machine

        **Return:**
            - ``output`` -- the contents of a CSV file
        """
        self.log.info('starting the ``_list_of_dictionaries_to_csv`` function')

        if not len(self.listOfDictionaries):
            return "NO MATCH"

        dataCopy = copy.deepcopy(self.listOfDictionaries)

        tableColumnNames = dataCopy[0].keys()
        columnWidths = []
        columnWidths[:] = [len(tableColumnNames[i])
                           for i in range(len(tableColumnNames))]

        output = io.BytesIO()
        # setup csv styles
        if csvType == "machine":
            delimiter = ","
        elif csvType in ["human", "markdown"]:
            delimiter = "|"
        writer = csv.writer(output, dialect='excel', delimiter=delimiter,
                            quotechar='"', quoting=csv.QUOTE_MINIMAL)

        if csvType in ["markdown"]:
            dividerWriter = csv.writer(output, dialect='excel', delimiter="|",
                                       quotechar='"', quoting=csv.QUOTE_MINIMAL)
        else:
            dividerWriter = csv.writer(output, dialect='excel', delimiter="+",
                                       quotechar='"', quoting=csv.QUOTE_MINIMAL)
        # add column names to csv
        header = []
        divider = []
        allRows = []

        # clean up data
        for row in dataCopy:
            for c in tableColumnNames:
                if isinstance(row[c], float) or isinstance(row[c], long) or isinstance(row[c], Decimal):
                    row[c] = "%0.4f" % row[c]
                elif isinstance(row[c], datetime):
                    thisDate = str(row[c])[:10]
                    row[c] = "%(thisDate)s" % locals()

        # set the column widths
        for row in dataCopy:
            for i, c in enumerate(tableColumnNames):
                if len(str(row[c])) > columnWidths[i]:
                    columnWidths[i] = len(str(row[c]))

        # fill in the data
        for row in dataCopy:
            thisRow = []
            # table border for human readable
            if csvType in ["human", "markdown"]:
                thisRow.append("")

            for i, c in enumerate(tableColumnNames):
                if csvType in ["human", "markdown"]:
                    if row[c] == None:
                        row[c] = ""
                    row[c] = str(str(row[c]).ljust(columnWidths[i] + 2)
                                 .rjust(columnWidths[i] + 3))
                thisRow.append(row[c])
            # table border for human readable
            if csvType in ["human", "markdown"]:
                thisRow.append("")
            allRows.append(thisRow)

        # table borders for human readable
        if csvType in ["human", "markdown"]:
            header.append("")
            divider.append("")

        for i, c in enumerate(tableColumnNames):
            if csvType == "machine":
                header.append(c)
            elif csvType in ["human", "markdown"]:
                header.append(
                    c.ljust(columnWidths[i] + 2).rjust(columnWidths[i] + 3))
                divider.append('-' * (columnWidths[i] + 3))

        # table border for human readable
        if csvType in ["human", "markdown"]:
            header.append("")
            divider.append("")

        if csvType == "machine":
            writer.writerow(header)
        if csvType in ["human"]:
            dividerWriter.writerow(divider)
            writer.writerow(header)
            dividerWriter.writerow(divider)
        elif csvType in ["markdown"]:
            writer.writerow(header)
            dividerWriter.writerow(divider)

        # write out the data
        writer.writerows(allRows)
        # table border for human readable
        if csvType in ["human"]:
            dividerWriter.writerow(divider)

        output = output.getvalue()
        output = output.strip()

        if csvType in ["markdown"]:
            output = output.replace("|--", "|:-")

        self.log.info(
            'completed the ``_list_of_dictionaries_to_csv`` function')

        return output

    def _list_of_dictionaries_to_mysql_inserts(
            self,
            tableName,
            createStatement=None):
        """Convert a python list of dictionaries to pretty csv output

        **Key Arguments:**
            - ``tableName`` -- the name of the table to create the insert statements for
            - ``createStatement`` -- add this create statement to the top of the file. Will only be executed if no table of that name exists in database. Default *None*

        **Return:**
            - ``output`` -- the mysql insert statements (as a string)
        """
        self.log.info(
            'starting the ``_list_of_dictionaries_to_mysql_inserts`` function')

        if not len(self.listOfDictionaries):
            return "NO MATCH"

        dataCopy = copy.deepcopy(self.listOfDictionaries)

        if createStatement:
            output = createStatement + "\n"
        else:
            output = ""

        for d in dataCopy:
            insertCommand = convert_dictionary_to_mysql_table(
                log=self.log,
                dictionary=d,
                dbTableName=tableName,
                uniqueKeyList=[],
                dateModified=False,
                returnInsertOnly=True,
                replace=True,
                batchInserts=False
            )
            output += insertCommand + ";\n"

        self.log.info(
            'completed the ``_list_of_dictionaries_to_mysql_inserts`` function')
        return output



import os
import nose
import shutil
import yaml

from fundamentals.utKit import utKit

from fundamentals import tools

su = tools(
    arguments={"settingsFile": None},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName="fundamentals",
    tunnel=False
)
arguments, settings, log, dbConn = su.setup()

# load settings
stream = file(
    "/Users/Dave/.config/fundamentals/fundamentals.yaml", 'r')
settings = yaml.load(stream)
stream.close()

# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()


class test_convert_dictionary_to_mysql_table():

    def test_convert_dictionary_to_mysql_table_function(self):
        dictionary = {"a newKey": "cool", "and another": "super cool",
                      "uniquekey1": "cheese", "uniqueKey2": "burgers"}
        from fundamentals.mysql import convert_dictionary_to_mysql_table
        message = convert_dictionary_to_mysql_table(
            dbConn=dbConn,
            log=log,
            dictionary=dictionary,
            dbTableName="testing_table",
            uniqueKeyList=["uniquekey1", "uniqueKey2"],
            createHelperTables=False,
            dateModified=False,
            returnInsertOnly=False
        )

    def test_return_inserts(self):
        dictionary = {"a newKey": "cool", "and another": "super cool",
                      "uniquekey1": "cheese", "uniqueKey2": "burgers"}
        from fundamentals.mysql import convert_dictionary_to_mysql_table
        message = convert_dictionary_to_mysql_table(
            dbConn=dbConn,
            log=log,
            dictionary=dictionary,
            dbTableName="testing_table",
            uniqueKeyList=["uniquekey1", "uniqueKey2"],
            createHelperTables=False,
            dateModified=False,
            returnInsertOnly=True
        )

        print message

    def test_return_inserts_non_batch(self):
        dictionary = {"a newKey": "cool", "and another": "super cool",
                      "uniquekey1": "cheese", "uniqueKey2": "burgers"}
        from fundamentals.mysql import convert_dictionary_to_mysql_table
        inserts = convert_dictionary_to_mysql_table(
            dbConn=dbConn,
            log=log,
            dictionary=dictionary,
            dbTableName="testing_table",
            uniqueKeyList=["uniquekey1", "uniqueKey2"],
            dateModified=False,
            returnInsertOnly=True,
            replace=True,
            batchInserts=False
        )

        print inserts

    # def test_convert_dictionary_to_mysql_table_function_exception(self):

    #     from fundamentals import convert_dictionary_to_mysql_table
    #     try:
    #         this = convert_dictionary_to_mysql_table(
    #             log=log,
    #             settings=settings,
    #             fakeKey="break the code"
    #         )
    #         this.get()
    #         assert False
    #     except Exception, e:
    #         assert True
    #         print str(e)

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Execute a MySQL write query on a database table*

:Author:
    David Young

:Date Created:
    June 21, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
import MySQLdb
import warnings
warnings.filterwarnings('error', category=MySQLdb.Warning)


def writequery(
    log,
    sqlQuery,
    dbConn,
    Force=False,
    manyValueList=False
):
    """*Execute a MySQL write command given a sql query*

    **Key Arguments:**
        - ``sqlQuery`` -- the MySQL command to execute
        - ``dbConn`` -- the db connection
        - ``Force`` -- do not exit code if error occurs, move onto the next command
        - ``manyValueList`` -- a list of value tuples if executing more than one insert

    **Return:**
        - ``message`` -- error/warning message

    **Usage:**

        Here's an example of how to create a table using the database connection passed to the function:

        .. code-block:: python 

            from fundamentals.mysql import writequery
            sqlQuery = "CREATE TABLE `testing_table` (`id` INT NOT NULL, PRIMARY KEY (`id`))"
            message = writequery(
                log=log,
                sqlQuery=sqlQuery,
                dbConn=dbConn,
                Force=False,
                manyValueList=False
            )

        Here's a many value insert example:

        .. code-block:: python 

            from fundamentals.mysql import writequery
            sqlQuery = "INSERT INTO testing_table (id) values (%s)"
            message = writequery(
                log=log,
                sqlQuery=sqlQuery,
                dbConn=dbConn,
                Force=False,
                manyValueList=[(1,), (2,), (3,), (4,), (5,), (6,), (7,),
                               (8,), (9,), (10,), (11,), (12,), ]
            )

    """
    log.info('starting the ``writequery`` function')

    message = ""
    try:
        cursor = dbConn.cursor(MySQLdb.cursors.DictCursor)
    except Exception as e:
        log.error('could not create the database cursor.')
    # EXECUTE THE SQL COMMAND
    try:
        if manyValueList == False:
            cursor.execute(sqlQuery)
        else:
            # cursor.executemany(sqlQuery, manyValueList)
            # INSET LARGE LISTS IN BATCHES TO STOP MYSQL SERVER BARFING
            batch = 100000
            offset = 0
            stop = 0
            while stop == 0:
                thisList = manyValueList[offset:offset + batch]
                offset += batch
                a = len(thisList)
                cursor.executemany(sqlQuery, thisList)
                if len(thisList) < batch:
                    stop = 1

    except MySQLdb.Error as e:
        if e[0] == 1050 and 'already exists' in e[1]:
            log.info(str(e) + '\n')
        elif e[0] == 1062:
                           # Duplicate Key error
            log.debug('Duplicate Key error: %s' % (str(e), ))
            message = "duplicate key error"
        elif e[0] == 1061:
                           # Duplicate Key error
            log.debug('index already exists: %s' % (str(e), ))
            message = "index already exists"
        else:
            sqlQueryTrim = sqlQuery[:1000]
            message = 'MySQL write command not executed for this query: << %s >>\nThe error was: %s ' % (sqlQuery,
                                                                                                         str(e))
            if Force == False:
                log.error(message)
                sys.exit(0)
            else:
                log.warning(message)
                return -1
    except Exception as e:
        if "truncated" in str(e):
            log.error('%s\n Here is the sqlquery:\n%s' % (str(e), sqlQuery))
            if manyValueList:
                log.error('... and the values:\n%s' % (thisList, ))
        else:
            sqlQuery = sqlQuery[:2000]
            log.error(
                'MySQL write command not executed for this query: << %s >>\nThe error was: %s ' %
                (sqlQuery, str(e)))
            if Force == False:
                sys.exit(0)
            return -1
    # CLOSE THE CURSOR
    cOpen = True
    count = 0
    while cOpen:
        try:
            cursor.close()
            cOpen = False
        except Exception as e:
            time.sleep(1)
            count += 1
            if count == 10:
                log.warning('could not close the db cursor ' + str(e) + '\n')
                raise e
                count = 0

    log.info('completed the ``writequery`` function')
    return message



from setuptools import setup, find_packages
import os

moduleDirectory = os.path.dirname(os.path.realpath(__file__))
exec(open(moduleDirectory + "/fundamentals/__version__.py").read())


def readme():
    with open(moduleDirectory + '/README.rst') as f:
        return f.read()

setup(
    name='fundamentals',
    version=__version__,
    description='Some project setup tools including logging, settings and database connections',
    long_description=readme(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Utilities',
    ],
    keywords=['logging', 'database'],
    url='https://github.com/thespacedoctor/fundamentals',
    download_url='https://github.com/thespacedoctor/fundamentals/archive/v%(__version__)s.zip' % locals(
    ),
    author='David Young',
    author_email='davidrobertyoung@gmail.com',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'pyyaml',
        'docopt',
        'MySQL-python',
        'python-dateutil',
        'eventlet'
    ],
    test_suite='nose.collector',
    tests_require=['nose', 'nose-cover3'],
    entry_points={
        'console_scripts': ['mysqlSucker=fundamentals.mysql.directory_script_runner:main'],
    },
    zip_safe=False
)



#!/usr/local/bin/python
# encoding: utf-8
"""
*Logger setup for python projects*

:Author:
    David Young

:Date Created:
    October 10, 2012
"""
import os
import sys
import logging
from logging import handlers


def console_logger(
    level="INFO"
):
    """
    *Setup and return a console logger*

    **Key Arguments:**
        - ``level`` -- the level of logging required

    **Return:**
        - ``logger`` -- the console logger

    **Usage:**
        .. code-block:: python 

            from fundamentals import logs
            log = logs.console_logger(
                level="DEBUG"
            )
            log.debug("Testing console logger")
    """
    ################ > IMPORTS ################
    ## STANDARD LIB ##
    import logging
    import logging.config
    ## THIRD PARTY ##
    import yaml
    ## LOCAL APPLICATION ##

    # SETUP LOGGING
    loggerConfig = """
  version: 1
  formatters:
      console_style:
          format: '* %(asctime)s - %(levelname)s: %(pathname)s:%(funcName)s:%(lineno)d > %(message)s'
          datefmt: '%H:%M:%S'
  handlers:
      console:
          class: logging.StreamHandler
          level: """ + level + """
          formatter: console_style
          stream: ext://sys.stdout
  root:
      level: """ + level + """
      handlers: [console]"""

    logging.config.dictConfig(yaml.load(loggerConfig))
    logger = logging.getLogger(__name__)

    return logger


def setup_dryx_logging(yaml_file):
    """
    *setup dryx style python logging*

    **Key Arguments:**
        - ``level`` -- the level of logging required

    **Return:**
        - ``logger`` -- the console logger

    **Usage:**
        .. code-block:: python 

            from fundamentals import logs
            log = logs.setup_dryx_logging(
                yaml_file="/Users/Dave/.config/fundamentals/fundamentals.yaml"
            )
            log.error("test error")

        Here is an example of the settings in the yaml file:

        .. code-block:: yaml

            version: 1

            logging settings:
                formatters:
                    file_style:
                        format: '* %(asctime)s - %(name)s - %(levelname)s (%(pathname)s > %(funcName)s > %(lineno)d) - %(message)s  '
                        datefmt: '%Y/%m/%d %H:%M:%S'
                    console_style:
                        format: '* %(asctime)s - %(levelname)s: %(pathname)s:%(funcName)s:%(lineno)d > %(message)s'
                        datefmt: '%H:%M:%S'
                    html_style:
                        format: '<div id="row" class="%(levelname)s"><span class="date">%(asctime)s</span>   <span class="label">file:</span><span class="filename">%(filename)s</span>   <span class="label">method:</span><span class="funcName">%(funcName)s</span>   <span class="label">line#:</span><span class="lineno">%(lineno)d</span> <span class="pathname">%(pathname)s</span>  <div class="right"><span class="message">%(message)s</span><span class="levelname">%(levelname)s</span></div></div>'
                        datefmt: '%Y-%m-%d <span class= "time">%H:%M <span class= "seconds">%Ss</span></span>'
                handlers:
                    console:
                        class: logging.StreamHandler
                        level: DEBUG
                        formatter: console_style
                        stream: ext://sys.stdout
                    file:
                        class: logging.handlers.GroupWriteRotatingFileHandler
                        level: WARNING
                        formatter: file_style
                        filename: /Users/Dave/.config/fundamentals/fundamentals.log
                        mode: w+
                        maxBytes: 102400
                        backupCount: 1
                root:
                    level: WARNING
                    handlers: [file,console]

    """
    import logging
    import logging.config
    import yaml

    # IMPORT CUSTOM HANDLER THAT ALLOWS GROUP WRITING
    handlers.GroupWriteRotatingFileHandler = GroupWriteRotatingFileHandler

    # IMPORT YAML LOGGING DICTIONARY
    stream = file(yaml_file, 'r')
    yamlContent = yaml.load(stream)
    stream.close()

    # USE THE LOGGING SETTINGS SECTION OF THE SETTINGS DICTIONARY FILE IF THERE IS ONE
    # OTHERWISE ASSUME THE FILE CONTAINS ONLY LOGGING SETTINGS
    if "logging settings" in yamlContent:
        yamlContent = yamlContent["logging settings"]
        yamlContent["version"] = 1

    logging.config.dictConfig(yamlContent)
    # SET THE ROOT LOGGER
    logger = logging.getLogger(__name__)

    logging.captureWarnings(True)

    return logger


class GroupWriteRotatingFileHandler(handlers.RotatingFileHandler):

    """
    *rotating file handler for logging*
    """

    def doRollover(self):
        """
        *Override base class method to make the new log file group writable.*
        """
        # Rotate the file first.
        handlers.RotatingFileHandler.doRollover(self)

        # Add group write to the current permissions.
        currMode = os.stat(self.baseFilename).st_mode
        os.chmod(self.baseFilename, currMode | stat.S_IWGRP |
                 stat.S_IRGRP | stat.S_IWOTH | stat.S_IROTH)


class GroupWriteRotatingFileHandler(handlers.RotatingFileHandler):

    """
    *rotating file handler for logging*
    """

    def _open(self):
        prevumask = os.umask(0)
        rtv = logging.handlers.RotatingFileHandler._open(self)
        os.umask(prevumask)
        return rtv
